/**
 * @file
 * @brief Main Window class declaration
 */
#ifndef MAINWINDOW_HPP
#define MAINWINDOW_HPP

#include "RQ_OBJECT.h"
#include "TGFrame.h"
#include "TMutex.h"
#include "TTimer.h"
class TGCheckButton;
class TGComboBox;
class TGLabel;
class TGHProgressBar;
class TGSpeedo;
class TGHtml;
class TGTab;
class TGToolBar;
class TGNumberEntry;
class TGPictureButton;
class TRootEmbeddedCanvas;
class TImage;
class PulseSurfer;

class ThSFMC01;
#ifdef ZDTUSB
class Keysight;
#endif

/**
 * @brief The MainWindow class
 */
class MainWindow : public TGMainFrame
{

public:
    /**
     * @brief MainWindow
     * @param p pointer to parent
     * @param w width
     * @param h height
     * @param ps a pointer to a pulse surfer window. owned then by this class
     * @param vendor the vendor id for a Keysight pulse generator
     * @param prod the product id for a Keysight pulse generator
     * @param quit save and exit after DAQ thread stops (once started)
     */
    MainWindow(const TGWindow *const p, const UInt_t w, const UInt_t h, PulseSurfer* const ps=nullptr, const uint32_t vendor=0, const uint32_t prod=0, const bool quit=false);
    virtual ~MainWindow();///< Destructor

    MainWindow(const MainWindow& w) = delete;///< @param w window

    MainWindow& operator=(const MainWindow& w) = delete;///< @param w window @return object by reference

    void UpdateClock();                 ///< Slot to update clock in the GUI
    void UpdateLabels();                ///< Slot to update labels in the GUI
    void UpdateSensors();               ///< Slot to update sensor speedometers in the GUI
    void UpdateRegisters(Int_t t=-1);   ///< Slot to update register table in the GUI \param t the selected tab index
    void UpdateScreenshot();            ///< Slot to update Keysight screenshot

    /**
     * @brief Slot function to update DAQ monitors in the GUI
     * @param ch if channel A or channel B was read (0 or 1)
     * @param cNum the number of chunks already read for this channel
     * @param writePos the current position of the RAM write pointer (address offset in bytes)
     * @param dFill the fill status (in bytes) of the DMA of the posterior chunk (before having read the current)
     * @param readTime the time it took to read the chunk, in microseconds
     * @param startTime the time point when the readout started, in milliseconds
     */
    void UpdateMonitors(ULong64_t ch, ULong64_t cNum, ULong64_t writePos, ULong64_t dFill, ULong64_t readTime, ULong64_t startTime);

    /**
     * @brief Slot function to save file to disk and quit
     */
    void SaveAndExit();

    void HandleTabs(Int_t idx);     ///< Slot to control the update timers @param idx the currently selected tab
    void DoConfigureSampling();     ///< Slot to configure the SFMC sampling parameters and ADC clock
    void DoArm();                   ///< Slot to arm sampling logic
    void DoSoftwareTrigger();       ///< Slot to send a software trigger
    void DoDisarmLogic();           ///< Slot to disarm sampling logic
    void DoReserveMemory();         ///< Slot to allocate RAM memory
    void DoStartDAQ();              ///< Slot to start DAQ
    void DoPauseDAQ();              ///< Slot to pause DAQ
    void DoStopDAQ();               ///< Slot to stop DAQ
    void DoSaveData(const bool batch = false);///< Slot to save RAM to disk @param batch if true, do not show FileDialog
    void ToggleSurfer();            ///< Slot to toggle Pulse Surfer channel
    void DoClose();                 ///< Slot to delete objects before ending

    void DoKeysightTrigger();       ///< Slot to send a trigger to the pulse generator

    /**
     * @brief Set the name of the default output folder
     * @param folder the output path
     */
    void SetOutputFolder(const TString folder);

    /**
     * @brief Setup the PRBS test protocol
     * @param pol the generator polynominal, PN3 through PN32
     * @param dptc if true, use DPTC compression of ADC data
     * @param gz if true, compress as gz file when saving to HDD
     */
    void SetupPRBS(const TString pol, const bool dptc, const bool gz);
    void AcquirePRBS();///< Perform data acquisition of PRBS

    /**
     * @brief Setup the ADC test pattern
     * @param pat Normal operation (0), Midscale short (1),
     * Positive full scale (2), Negative full scale (3),
     * Checker board (4), Random long (5),
     * Random short (6), 1/0 word toggle (7),
     * User pattern (8), Ramp output (15)
     * @param dptc if true, enable ADC data stream compression using DPTC
     * @param gz if true, compress as gz file when saving to HDD
     */
    void SetupADCTest(const int pat, const bool dptc, const bool gz);
    void AcquireADCTest();///< Perform data acquisition of ADC test pattern

private:
    PulseSurfer* const fSurfer{};   ///< Pulse Surfer window pointer
    Short_t fSurferChannel;         ///< Current pulse surfer channel
    TGTab* fTab{};                  ///< GUI tab pointer
    TGTextButton* fConfigure{};     ///< Configure sampling button
    TGTextButton* fReserveMem{};    ///< Reserve memory button
    TGTextButton* fArming{};        ///< Arm logic button
    TGTextButton* fSwTrig{};        ///< Software trigger button
    TGTextButton* fDisarm{};        ///< Disarm logic button
    TGToolBar* fDAQControls{};      ///< DAQ control toolbar
    TGComboBox* fWriteCompression{};///< compress when writing to disk
    TGLabel* fOutFile{};            ///< output directory name

    TGCheckButton* fRisingEdge{};   ///< trigger on rising edge
    TGCheckButton* fFallingEdge{};  ///< trigger on falling edge
    TGCheckButton* fExtTrigDelay{}; ///< external trigger delay
    TGCheckButton* fSignedMode{};   ///< decimation/average signed mode
    TGComboBox* fFrequency{};       ///< Sampling frequency
    TGComboBox* fDecimation{};      ///< Decimation factor
    TGCheckButton* fAveraging{};    ///< sample averaging
    TGCheckButton* fEventHeader{};  ///< event header
    TGCheckButton* fContinuous{};   ///< continuous sampling
    TGCheckButton* fBinary{};       ///< binary ADC format
    TGComboBox* fTestMode{};        ///< ADC Test mode

    TGLabel* fArmed{};              ///< logic is armed
    TGLabel* fSamplingBusy{};       ///< sampling is active
    TGLabel* fBank1Busy{};          ///< Memory Bank 1 is busy
    TGLabel* fBank2Busy{};          ///< Memory Bank 2 is busy
    TGLabel* fLocked{};             ///< HMC and ADC PLLs are locked
    TGLabel* fOverrange{};          ///< Overrange status bits
    TGTextButton* fExit{};          ///< Exit button

    TGCheckButton* fStore[2];       ///< chA/chB storing
    TGComboBox* fSource[2];         ///< chA/chB data source
    TGCheckButton* fCompress[2];    ///< chA/chB data compression
    TGCheckButton* fTestCompress[2];///< chA/chB test compression mode
    TGCheckButton* fSaveGPIO[2];    ///< chA/chB store GPIO
    TGComboBox* fGPIOSelect[2];     ///< chA/chB GPIO channel selection
    TGComboBox* fStartBlock[2];     ///< chA/chB DMA start block address
    TGComboBox* fChunkSize[2];      ///< chA/chB DMA read chunk bytes
    TGSpeedo* fNextFill[2];         ///< chA/chB fill status of the next DMA chunk (N+1) buffer just before reading the current one (N).
    TGNumberEntry* fBufferSize[2];  ///< chA/chB RAM buffer bytes
    TGHProgressBar* fBufferFill[2]; ///< chA/chB fill status of the RAM buffer
    TGHProgressBar* fTransfSpeed[2];///< chA/chB DMA to RAM transfer speed

    TGLabel* fClock{};              ///< DAQ stopwatch
    TTimer fClockUpdater{};         ///< DAQ stopwatch update timer
    TTime fStartTime{};             ///< DAQ start time @todo maybe change to TTimeStamp
    TTime fPauseTime{};             ///< DAQ pause time @todo maybe change to TTimeStamp
    Long64_t fPausedTime{};         ///< DAQ paused time in millisec

    TGSpeedo* fSensors[17];         ///< temperature, voltage and ventilator sensors
    TTimer fSensorUpdater{};        ///< board sensor update timer
    TGHtml* fHtmlSis{};             ///< SIS1160 board register table
    TGHtml* fHtmlSfmc{};            ///< SFMC01 board register table
    TGHtml* fHtmlAdc{};             ///< AD9689 board register table
    TGHtml* fHtmlHmc{};             ///< HMC7044 board register table
    TGHtml* fHtmlSi570_1{};         ///< Si570 oscillator 1 board register table
    TGHtml* fHtmlSi570_2{};         ///< Si570 oscillator 2 board register table
    TGHtml* fHtmlSi5326{};          ///< Si5326 board register table
    TTimer fRegisterUpdater{};      ///< boards' registers update timer

    ThSFMC01* const fThCont;        ///< pointer to threaded SFMC01 controller class @note Accessible in the Terminal Plugin via variable fThCont
    bool fQuit;                     ///< save and exit after DAQ thread stops (once started)
    TTimer fQuitter{};              ///< deferred program quit
    TTimer fStarter{};              ///< deferred trigger start
    TMutex fGuiMutex{};             ///< GUI control mutex
    #ifdef ZDTUSB
    char* fPicBuffer{};             ///< Picture buffer
    size_t fPicBufferSize{};        ///< Picture buffer size. len(fPicBuffer). Cannot be just strlen() because there are NULL bytes.
    TImage* fPicture{};             ///< Keysight screenshot picture
    Keysight* const fKeysight;      ///< pointer to Keysight controller class @note Accessible in the Terminal Plugin via variable fKeysight
    TTimer fScreenshotUpdater{};    ///< Keysight's screenshot updater
    TRootEmbeddedCanvas* fScreenshot{};///<Keysight's screenshot canvas
    #endif
    uint32_t fDMASize[2];           ///< The DMA read chunk size in bytes for each channel
    uint64_t fRAMSize[2];           ///< The maximum buffer RAM size in bytes for each channel
    uint32_t* fBufferRAM[2];        ///< Buffer in the RAM where to store acquired ADC data, for each channel

    uint64_t fMaxBytes[2];          ///< Max bytes after whciht to stop storing data to disk for each channel. Default is 0, and disables this function.

#if ROOT_VERSION_CODE < ROOT_VERSION(6,0,0)//Just needed for ROOT v5
ClassDef(MainWindow,1);// we have to live with the compilation Warning in <MainWindow>: The data members of MainWindow will not be stored, because it inherits from TObject but does not have its own ClassDef.
#endif
};

#endif // MAINWINDOW_HPP
