/**
 * @file
 * @brief RThread class declaration
 */

#ifndef RTHREAD_HPP
#define RTHREAD_HPP

#include <string>
#include <atomic>

class TThread;//fw declaration
class TTimeStamp;//fw declaration

/**
 * @brief Base class for other classes that need to perform an action in a single separate thread
 *
 * Call Start() to start the ThreadFunction(), Stop() to finish it.
 * For simplicity, no priorities are taken into account.
 *
 * @note You can inherit from this class and implement the ThreadFunction() and the destructor. The latter should ideally call Stop()
 * @see https://web-docs.gsi.de/~go4/HOWTOthreads/howtothreadsbody.html#tth_sEc6.1
 * @see https://stackoverflow.com/questions/24206878/constructing-thread-executing-a-member-function-of-a-derived-class/24209679#24209679
 */
class RThread
{
public:
    /**
     * @brief Start the thread function
     * @return return code of TThread::Run
     */
    int Start();

    /**
     * @brief Pause the thread function
     * @return return code of TThread::Join
     */
    int Pause();

    /**
     * @brief Stop the thread function
     * @return return code of TThread::Join
     */
    int Stop();

    /**
     * @brief Join/finish the thread
     * @return 1 if thread pointer existed, 0 otherwise
     */
    int Join();

    /**
     * @brief Kill the thread
     * @return 1 if thread pointer existed, 0 otherwise
     */
    int Kill();

    /**
     * @brief Checks if the thread is running
     * @return 1 if running, 0 otherwise
     */
    int IsRunning() const;

    /**
     * @brief Put thread to sleep for some time
     * @param secs the number of seconds to sleep
     * @param nanos the number of nanoseconds to sleep
     */
    void Sleep(unsigned long secs, unsigned long nanos = 0);

protected:
    /**
     * @brief RThread constructor
     * @param name the name assigned to this thread
     */
    RThread(const std::string name);

    RThread(const RThread&) = delete;///< disable copy assignment

    RThread& operator=(const RThread&) = delete;///< disable assignment @return reference

    virtual ~RThread();///< Destructor

    /**
     * @brief Main thread function, to be implemented in child class
     *
     * Function will end when calling stop if the ThreadFunction checks the value of fThStopFlag regularly in the loop.
     *
     * @note Code in this function will be run as a TThread after calling Start()
     */
    virtual void ThreadFunction();

    std::atomic<bool> fStopFlag;         ///<Flag to indicate that the ThreadFunction should exit.
    std::atomic<bool> fPauseFlag;        ///<Flag to indicate that the ThreadFunction should pause.

    const std::string fName;///< Name of the thread

    TTimeStamp* fStart{};   ///< Timestamp
    TTimeStamp* fStop{};    ///< Timestamp

private:
    /**
     * @brief Signature of the static functor need for TThread
     * @param ptr will be a pointer to this same class
     * @return
     */
    static void *ThreadHandle(void *ptr);

    TThread* fThread;   ///< a Pointer to the internal TThread
};

#endif
