/**
 * @file
 * SFMC01+SIS1160 threaded controller class declaration
 */

#ifndef THSFMC01_HPP
#define THSFMC01_HPP

#include "TMutex.h"
#include <RQ_OBJECT.h>
#include "RThread.hpp"

/**
 * @brief The SFMC01 threaded controller class
 *
 * It is a thread-safe wrapper class around the SFMC01 class from which it derives
 *
 */
class ThSFMC01 : public RThread
{
    RQ_OBJECT("ThSFMC01")
public:
    /**
     * @brief Threaded SFMC01 constructor, opens the device
     * @param deviceNr the id of the device, usually 0. Should be < SFMC01::GetNumberOfDevices
     * @note If an error is found, fValid is set to false and fDevice to nullptr
     */
    ThSFMC01(const uint16_t deviceNr);

    virtual ~ThSFMC01();

    //TMutex fMutex{};                ///< SFMC communication mutex

    static uint32_t kDMASize[2];    ///< The static DMA read chunk size in bytes for each channel
    static uint64_t kRAMSize[2];    ///< The static maximum buffer RAM size in bytes for each channel
    static uint32_t* kBufferRAM[2]; ///< Static Buffer in the RAM where to store acquired ADC data, for each channel

    /**
     * @brief Signal emitted when a chunk is read
     * @param ch if channel A or channel B was read (0 or 1)
     * @param cNum the number of chunks already read for this channel
     * @param writePos the current position of the RAM write pointer (address offset in bytes)
     * @param dFill the fill status (in bytes) of the DMA of the posterior chunk (before having read the current)
     * @param readTime the time it took to read the chunk, in microseconds
     * @param startTime the time point when the readout started, in milliseconds
     */
    virtual void ChunkRead(ULong64_t ch, ULong64_t cNum, ULong64_t writePos, ULong64_t dFill, ULong64_t readTime, ULong64_t startTime);// *SIGNAL*

    /**
     * @brief Signal emitted when DAQ thread finishes
     */
    virtual void Finished();// *SIGNAL*

    /**
     * @brief Readout Thread
     */
    void ThreadFunction();
};

#endif // THSFMC01_HPP
